import {
  ChangeDetectionStrategy,
  Component,
  ViewChild,
} from "@angular/core";
import {MatMenu} from "@angular/material/menu";
import {SharedAbstractPresentational} from "@app/shared/components/presentationals/shared-abstract/shared-abstract.presentational";
import {Store} from "@ngxs/store";
import {LabelTranslateEnum} from "@shared/enums/label-translate.enum";
import {RoutesEnum} from "@shared/enums/routes.enum";
import {StepTourSectionNameEnum} from "@shared/enums/step-tour-section-name.enum";
import {AppTourService} from "@shared/services/app-tour.service";
import {IStepOption} from "ngx-ui-tour-md-menu";
import {MARK_AS_TRANSLATABLE} from "solidify-frontend";

@Component({
  selector: "dlcm-guided-tour-menu",
  templateUrl: "./guided-tour-menu.presentational.html",
  styleUrls: ["./guided-tour-menu.presentational.scss"],
  changeDetection: ChangeDetectionStrategy.OnPush,
  exportAs: "menuGuidedTour",
})
export class GuidedTourMenuPresentational extends SharedAbstractPresentational {
  @ViewChild(MatMenu, {static: true}) menu: MatMenu;

  listMenu: Menu[] = [
    {
      titleToTranslate: MARK_AS_TRANSLATABLE("tour.name.main"),
      stepTourSectionName: StepTourSectionNameEnum.main,
    },
    {
      titleToTranslate: LabelTranslateEnum.deposit,
      stepTourSectionName: StepTourSectionNameEnum.deposit,
      navigationEnd: RoutesEnum.deposit,
    },
    {
      titleToTranslate: MARK_AS_TRANSLATABLE("tour.name.depositMetadata"),
      stepTourSectionName: StepTourSectionNameEnum.depositMetadata,
      navigationEnd: RoutesEnum.deposit,
    },
    {
      titleToTranslate: MARK_AS_TRANSLATABLE("tour.name.depositData"),
      stepTourSectionName: StepTourSectionNameEnum.depositData,
      navigationEnd: RoutesEnum.deposit,
    },
    {
      titleToTranslate: LabelTranslateEnum.preservationSpace,
      stepTourSectionName: StepTourSectionNameEnum.preservationSpace,
    },
    {
      titleToTranslate: LabelTranslateEnum.organizationalUnit,
      stepTourSectionName: StepTourSectionNameEnum.organizationalUnit,
      navigationEnd: RoutesEnum.preservationSpaceOrganizationalUnit,
    },
    {
      titleToTranslate: LabelTranslateEnum.search,
      stepTourSectionName: StepTourSectionNameEnum.search,
    },
  ];

  constructor(private readonly _appTourService: AppTourService,
              private readonly _store: Store) {
    super();
  }

  runTour(menu: Menu): void {
    this._appTourService.runTour(menu.stepTourSectionName, menu.extraOptions, menu.navigationEnd);
  }
}

interface Menu {
  titleToTranslate: string;
  stepTourSectionName: StepTourSectionNameEnum;
  extraOptions?: IStepOption | undefined;
  navigationEnd?: string;
}
