import {
  ChangeDetectionStrategy,
  ChangeDetectorRef,
  Component,
  ElementRef,
  Injector,
} from "@angular/core";
import {
  FormBuilder,
  Validators,
} from "@angular/forms";
import {Enums} from "@enums";
import {DisseminationPolicy} from "@models";
import {IconNameEnum} from "@shared/enums/icon-name.enum";
import {LabelTranslateEnum} from "@shared/enums/label-translate.enum";
import {BaseFormDefinition} from "@shared/models/base-form-definition.model";
import {
  AbstractFormPresentational,
  EnumUtil,
  PropertyName,
  SolidifyValidator,
} from "solidify-frontend";

@Component({
  selector: "dlcm-admin-dissemination-policy-form",
  templateUrl: "./admin-dissemination-policy-form.presentational.html",
  styleUrls: ["./admin-dissemination-policy-form.presentational.scss"],
  changeDetection: ChangeDetectionStrategy.OnPush,
})
export class AdminDisseminationPolicyFormPresentational extends AbstractFormPresentational<DisseminationPolicy> {
  formDefinition: FormComponentFormDefinition = new FormComponentFormDefinition();

  disseminationPolicyTypeEnum: Enums.DisseminationPolicy.TypeEnum[] = EnumUtil.convertToArray(Enums.DisseminationPolicy.TypeEnum);

  get labelTranslateEnum(): typeof LabelTranslateEnum {
    return LabelTranslateEnum;
  }

  get iconNameEnum(): typeof IconNameEnum {
    return IconNameEnum;
  }

  constructor(protected readonly _changeDetectorRef: ChangeDetectorRef,
              protected readonly _elementRef: ElementRef,
              protected readonly _injector: Injector,
              private readonly _fb: FormBuilder) {
    super(_changeDetectorRef, _elementRef, _injector);
  }

  protected bindFormTo(disseminationPolicies: DisseminationPolicy): void {
    this.form = this._fb.group({
      [this.formDefinition.name]: [disseminationPolicies.name, [Validators.required, SolidifyValidator]],
      [this.formDefinition.destinationServer]: [disseminationPolicies.destinationServer, [SolidifyValidator]],
      [this.formDefinition.subFolder]: [disseminationPolicies.subFolder, [SolidifyValidator]],
      [this.formDefinition.type]: [disseminationPolicies.type, [SolidifyValidator]],
    });
  }

  protected initNewForm(): void {
    this.form = this._fb.group({
      [this.formDefinition.name]: ["", [Validators.required, SolidifyValidator]],
      [this.formDefinition.destinationServer]: ["", [Validators.required, SolidifyValidator]],
      [this.formDefinition.subFolder]: ["", [SolidifyValidator]],
      [this.formDefinition.type]: ["", [Validators.required, SolidifyValidator]],
    });
  }

  protected treatmentBeforeSubmit(disseminationPolicies: DisseminationPolicy): DisseminationPolicy {
    return disseminationPolicies;
  }

}

class FormComponentFormDefinition extends BaseFormDefinition {
  @PropertyName() name: string;
  @PropertyName() destinationServer: string;
  @PropertyName() subFolder: string;
  @PropertyName() type: string;
}
