import {
  AdminOaiSetAction,
  adminOaiSetActionNameSpace,
} from "@admin/oai-set/stores/admin-oai-set.action";
import {
  AdminOaiSetState,
  AdminOaiSetStateModel,
} from "@admin/oai-set/stores/admin-oai-set.state";
import {
  ChangeDetectionStrategy,
  ChangeDetectorRef,
  Component,
  Injector,
  ViewChild,
} from "@angular/core";
import {
  OaiSet,
  OrganizationalUnit,
} from "@models";
import {
  Actions,
  Select,
  Store,
} from "@ngxs/store";
import {StateEnum} from "@shared/enums/state.enum";
import {SharedOrganizationalUnitState} from "@shared/stores/organizational-unit/shared-organizational-unit.state";
import {Observable} from "rxjs";
import {
  AbstractCreateRoutable,
  AbstractFormPresentational,
  MemoizedUtil,
  ModelFormControlEvent,
} from "solidify-frontend";

@Component({
  selector: "dlcm-admin-oai-set-bulk-create-routable",
  templateUrl: "./admin-oai-set-bulk-create.routable.html",
  styleUrls: ["./admin-oai-set-bulk-create.routable.scss"],
  changeDetection: ChangeDetectionStrategy.OnPush,
})
export class AdminOaiSetBulkCreateRoutable extends AbstractCreateRoutable<OaiSet, AdminOaiSetStateModel> {
  @Select(AdminOaiSetState.isLoadingWithDependency) isLoadingWithDependencyObs: Observable<boolean>;
  @Select(AdminOaiSetState.isReadyToBeDisplayedInCreateMode) isReadyToBeDisplayedInCreateModeObs: Observable<boolean>;
  listOrgUnit: Observable<OrganizationalUnit[]> = MemoizedUtil.list(this._store, SharedOrganizationalUnitState);

  @ViewChild("formPresentational")
  readonly formPresentational: AbstractFormPresentational<OaiSet>;

  constructor(protected readonly _store: Store,
              protected readonly _actions$: Actions,
              protected readonly _changeDetector: ChangeDetectorRef,
              protected readonly _injector: Injector) {
    super(_store, _actions$, _changeDetector, StateEnum.admin_oaiSet, _injector, adminOaiSetActionNameSpace, StateEnum.admin);
  }

  create(modelFormControlEvent: ModelFormControlEvent<OaiSet>): void {
    super.saveInProgress();
    this._store.dispatch(new AdminOaiSetAction.BulkCreate(modelFormControlEvent.model.organizationalUnits));
  }
}
