import {
  ChangeDetectionStrategy,
  ChangeDetectorRef,
  Component,
  ElementRef,
  Injector,
} from "@angular/core";
import {
  FormBuilder,
  Validators,
} from "@angular/forms";
import {Enums} from "@enums";
import {environment} from "@environments/environment";
import {AdditionalFieldsForm} from "@models";
import {LabelTranslateEnum} from "@shared/enums/label-translate.enum";
import {BaseFormDefinition} from "@shared/models/base-form-definition.model";
import {
  AbstractFormPresentational,
  BreakpointService,
  KeyValue,
  PropertyName,
  SolidifyValidator,
} from "solidify-frontend";

@Component({
  selector: "dlcm-admin-organizational-unit-additional-fields-form",
  templateUrl: "./admin-organizational-unit-additional-fields-form.presentational.html",
  styleUrls: ["./admin-organizational-unit-additional-fields-form.presentational.scss"],
  changeDetection: ChangeDetectionStrategy.OnPush,
})
export class AdminOrganizationalUnitAdditionalFieldsFormPresentational extends AbstractFormPresentational<AdditionalFieldsForm> {
  readonly TIME_BEFORE_DISPLAY_TOOLTIP: number = environment.timeBeforeDisplayTooltip;
  formDefinition: FormComponentFormDefinition = new FormComponentFormDefinition();

  listFormType: KeyValue[] = Enums.FormDescription.LanguageEnumTranslate;

  get labelTranslateEnum(): typeof LabelTranslateEnum {
    return LabelTranslateEnum;
  }

  constructor(protected readonly _changeDetectorRef: ChangeDetectorRef,
              protected readonly _elementRef: ElementRef,
              protected readonly _injector: Injector,
              private readonly _fb: FormBuilder,
              public readonly breakpointService: BreakpointService) {
    super(_changeDetectorRef, _elementRef, _injector);
  }

  protected initNewForm(): void {
    this.form = this._fb.group({
      [this.formDefinition.type]: [Enums.FormDescription.TypeEnum.FORMLY, [Validators.required, SolidifyValidator]],
      [this.formDefinition.name]: ["", [SolidifyValidator]],
      [this.formDefinition.description]: [""],
    });

  }

  protected bindFormTo(additionalFieldsForm: AdditionalFieldsForm): void {
    this.form = this._fb.group({
      [this.formDefinition.type]: [additionalFieldsForm.type, [Validators.required, SolidifyValidator]],
      [this.formDefinition.name]: [additionalFieldsForm.name, [SolidifyValidator]],
      [this.formDefinition.description]: [additionalFieldsForm.description],
    });
  }

  protected treatmentBeforeSubmit(additionalFieldsForm: AdditionalFieldsForm | any): AdditionalFieldsForm {
    return additionalFieldsForm;
  }
}

class FormComponentFormDefinition extends BaseFormDefinition {
  @PropertyName() type: string;
  @PropertyName() name: string;
  @PropertyName() description: string;
  @PropertyName() creation: string;
  @PropertyName() lastUpdate: string;
}
