import {adminOrganizationalUnitActionNameSpace} from "@admin/organizational-unit/stores/admin-organizational-unit.action";
import {AdminOrganizationalUnitStateModel} from "@admin/organizational-unit/stores/admin-organizational-unit.state";
import {
  ChangeDetectionStrategy,
  ChangeDetectorRef,
  Component,
  Injector,
} from "@angular/core";
import {MatDialog} from "@angular/material/dialog";
import {ActivatedRoute} from "@angular/router";
import {OrganizationalUnit} from "@models";
import {Navigate} from "@ngxs/router-plugin";
import {
  Actions,
  Store,
} from "@ngxs/store";
import {DataTestEnum} from "@shared/enums/data-test.enum";
import {LabelTranslateEnum} from "@shared/enums/label-translate.enum";
import {
  AdminRoutesEnum,
  RoutesEnum,
} from "@shared/enums/routes.enum";
import {StateEnum} from "@shared/enums/state.enum";
import {
  AbstractListRoutable,
  DataTableFieldTypeEnum,
  OrderEnum,
  RouterExtensionService,
} from "solidify-frontend";

@Component({
  selector: "dlcm-admin-organizational-unit-list-routable",
  templateUrl: "../../../../../../../../node_modules/solidify-frontend/lib/components/routables/abstract-list/abstract-list.routable.html",
  styleUrls: ["./admin-organizational-unit-list.routable.scss"],
  changeDetection: ChangeDetectionStrategy.OnPush,
})
export class AdminOrganizationalUnitListRoutable extends AbstractListRoutable<OrganizationalUnit, AdminOrganizationalUnitStateModel> {
  readonly KEY_CREATE_BUTTON: string = LabelTranslateEnum.create;
  readonly KEY_BACK_BUTTON: string | undefined = LabelTranslateEnum.backToAdmin;
  readonly KEY_PARAM_NAME: keyof OrganizationalUnit & string = "name";

  constructor(protected readonly _store: Store,
              protected readonly _changeDetector: ChangeDetectorRef,
              protected readonly _route: ActivatedRoute,
              protected readonly _routerExt: RouterExtensionService,
              protected readonly _actions$: Actions,
              protected readonly _dialog: MatDialog,
              protected readonly _injector: Injector) {
    super(_store, _changeDetector, _route, _routerExt, _actions$, _dialog, StateEnum.admin_organizationalUnit, adminOrganizationalUnitActionNameSpace, _injector, {}, StateEnum.admin);
  }

  conditionDisplayEditButton(model: OrganizationalUnit | undefined): boolean {
    return true;
  }

  conditionDisplayDeleteButton(model: OrganizationalUnit | undefined): boolean {
    return true;
  }

  override goToEdit(model: OrganizationalUnit): void {
    this._store.dispatch(new Navigate([RoutesEnum.adminOrganizationalUnitDetail, model.resId, AdminRoutesEnum.organizationalUnitData, AdminRoutesEnum.organizationalUnitEdit], {}, {skipLocationChange: true}));
  }

  defineColumns(): void {
    this.columns = [
      {
        field: "name",
        header: LabelTranslateEnum.nameLabel,
        type: DataTableFieldTypeEnum.string,
        order: OrderEnum.none,
        isFilterable: true,
        isSortable: true,
        dataTest: DataTestEnum.adminOrgUnitListSearchName,
      },
      {
        field: "openingDate",
        header: LabelTranslateEnum.opening,
        type: DataTableFieldTypeEnum.date,
        order: OrderEnum.descending,
        isFilterable: false,
        isSortable: true,
      },
      {
        field: "closingDate" as any,
        header: LabelTranslateEnum.closing,
        type: DataTableFieldTypeEnum.date,
        order: OrderEnum.none,
        isFilterable: false,
        isSortable: true,
      },
    ];
  }
}
