import {
  ChangeDetectionStrategy,
  ChangeDetectorRef,
  Component,
  ElementRef,
  Injector,
} from "@angular/core";
import {
  FormBuilder,
  Validators,
} from "@angular/forms";
import {SubmissionPolicy} from "@models";
import {IconNameEnum} from "@shared/enums/icon-name.enum";
import {LabelTranslateEnum} from "@shared/enums/label-translate.enum";
import {BaseFormDefinition} from "@shared/models/base-form-definition.model";
import {
  AbstractFormPresentational,
  PropertyName,
  SolidifyValidator,
} from "solidify-frontend";

@Component({
  selector: "dlcm-admin-submission-policy-form",
  templateUrl: "./admin-submission-policy-form.presentational.html",
  styleUrls: ["./admin-submission-policy-form.presentational.scss"],
  changeDetection: ChangeDetectionStrategy.OnPush,
})
export class AdminSubmissionPolicyFormPresentational extends AbstractFormPresentational<SubmissionPolicy> {
  formDefinition: FormComponentFormDefinition = new FormComponentFormDefinition();

  get labelTranslateEnum(): typeof LabelTranslateEnum {
    return LabelTranslateEnum;
  }

  get iconNameEnum(): typeof IconNameEnum {
    return IconNameEnum;
  }

  constructor(protected readonly _changeDetectorRef: ChangeDetectorRef,
              protected readonly _elementRef: ElementRef,
              protected readonly _injector: Injector,
              private readonly _fb: FormBuilder) {
    super(_changeDetectorRef, _elementRef, _injector);
  }

  protected initNewForm(): void {
    this.form = this._fb.group({
      [this.formDefinition.name]: ["", [Validators.required, SolidifyValidator]],
      [this.formDefinition.submissionApproval]: [false, [SolidifyValidator]],
      [this.formDefinition.timeToKeep]: [0, [SolidifyValidator]],
    });
  }

  protected bindFormTo(submissionPolicies: SubmissionPolicy): void {
    this.form = this._fb.group({
      [this.formDefinition.name]: [submissionPolicies.name, [Validators.required, SolidifyValidator]],
      [this.formDefinition.submissionApproval]: [submissionPolicies.submissionApproval, [SolidifyValidator]],
      [this.formDefinition.timeToKeep]: [submissionPolicies.timeToKeep, [SolidifyValidator]],
    });
  }

  protected treatmentBeforeSubmit(submissionPolicies: SubmissionPolicy): SubmissionPolicy {
    return submissionPolicies;
  }
}

class FormComponentFormDefinition extends BaseFormDefinition {
  @PropertyName() name: string;
  @PropertyName() submissionApproval: string;
  @PropertyName() timeToKeep: string;
}
