import {HttpClient} from "@angular/common/http";
import {Injectable} from "@angular/core";
import {DepositStateModel} from "@deposit/stores/deposit.state";
import {
  Action,
  Actions,
  State,
  StateContext,
  Store,
} from "@ngxs/store";
import {
  PreservationAipStatusAction,
  preservationAipStatusActionNameSpace,
} from "@preservation-planning/aip-status/stores/preservation-planning-aip-status.action";
import {ApiEnum} from "@shared/enums/api.enum";
import {urlSeparator} from "@shared/enums/routes.enum";
import {StateEnum} from "@shared/enums/state.enum";
import {AipCopyList} from "@shared/models/business/aip-copy-list.model";
import {Observable} from "rxjs";
import {
  catchError,
  tap,
} from "rxjs/operators";
import {
  ApiService,
  defaultResourceStateInitValue,
  MARK_AS_TRANSLATABLE,
  NotificationService,
  ResourceState,
  ResourceStateModel,
  SolidifyHttpErrorResponseModel,
  SolidifyStateError,
} from "solidify-frontend";
import {ApiActionNameEnum} from "../../../../shared/enums/api-action-name.enum";

export interface PreservationPlanningAipStatusStateModel extends ResourceStateModel<AipCopyList> {
}

@Injectable()
@State<PreservationPlanningAipStatusStateModel>({
  name: StateEnum.preservationPlanning_aipStatus,
  defaults: {
    ...defaultResourceStateInitValue(),
  },
})
export class PreservationPlanningAipStatusState extends ResourceState<PreservationPlanningAipStatusStateModel, AipCopyList> {
  constructor(protected apiService: ApiService,
              protected store: Store,
              protected notificationService: NotificationService,
              protected actions$: Actions,
              protected httpClient: HttpClient) {
    super(apiService, store, notificationService, actions$, {
      nameSpace: preservationAipStatusActionNameSpace,
    });
  }

  protected get _urlResource(): string {
    return ApiEnum.preservationPlanningPreservationAipStatuses;
  }

  @Action(PreservationAipStatusAction.DeleteCache)
  deleteCache(ctx: StateContext<DepositStateModel>, action: PreservationAipStatusAction.DeleteCache): Observable<string> {
    ctx.patchState({
      isLoadingCounter: ctx.getState().isLoadingCounter + 1,
    });

    return this.httpClient.post(this._urlResource + urlSeparator + ApiActionNameEnum.DELETE_CACHE, null, {
      responseType: "text",
    }).pipe(
      tap(result => {
        ctx.dispatch(new PreservationAipStatusAction.DeleteCacheSuccess(action));
      }),
      catchError((error: SolidifyHttpErrorResponseModel) => {
        ctx.dispatch(new PreservationAipStatusAction.DeleteCacheFail(action));
        throw new SolidifyStateError(this, error);
      }),
    );
  }

  @Action(PreservationAipStatusAction.DeleteCacheSuccess)
  deleteCacheSuccess(ctx: StateContext<DepositStateModel>, action: PreservationAipStatusAction.DeleteCacheSuccess): void {
    ctx.patchState({
      isLoadingCounter: ctx.getState().isLoadingCounter - 1,
    });
    this.notificationService.showSuccess(MARK_AS_TRANSLATABLE("admin.preservation.aipStatuses.notification.deleteCache.success"));
  }

  @Action(PreservationAipStatusAction.DeleteCacheFail)
  deleteCacheFail(ctx: StateContext<DepositStateModel>, action: PreservationAipStatusAction.DeleteCacheFail): void {
    ctx.patchState({
      isLoadingCounter: ctx.getState().isLoadingCounter - 1,
    });
    this.notificationService.showError(MARK_AS_TRANSLATABLE("admin.preservation.aipStatuses.notification.deleteCache.fail"));
  }
}
