import {Injectable} from "@angular/core";
import {
  State,
  Store,
} from "@ngxs/store";
import {
  PreservationPlanningAipStatusState,
  PreservationPlanningAipStatusStateModel,
} from "@preservation-planning/aip-status/stores/preservation-planning-aip-status.state";
import {
  PreservationPlanningDepositState,
  PreservationPlanningDepositStateModel,
} from "@preservation-planning/deposit/stores/preservation-planning-deposit.state";
import {
  PreservationPlanningDipState,
  PreservationPlanningDipStateModel,
} from "@preservation-planning/dip/stores/preservation-planning-dip.state";
import {
  PreservationPlanningJobState,
  PreservationPlanningJobStateModel,
} from "@preservation-planning/job/stores/preservation-planning-job.state";
import {
  PreservationPlanningMonitoringState,
  PreservationPlanningMonitoringStateModel,
} from "@preservation-planning/monitoring/stores/preservation-planning-monitoring.state";
import {
  PreservationPlanningSipState,
  PreservationPlanningSipStateModel,
} from "@preservation-planning/sip/stores/preservation-planning-sip.state";
import {StateEnum} from "@shared/enums/state.enum";
import {BaseStateModel} from "solidify-frontend";

export interface PreservationPlanningStateModel extends BaseStateModel {
  [StateEnum.preservationPlanning_deposit]: PreservationPlanningDepositStateModel | undefined;
  [StateEnum.preservationPlanning_monitoring]: PreservationPlanningMonitoringStateModel | undefined;
  [StateEnum.preservationPlanning_job]: PreservationPlanningJobStateModel | undefined;
  [StateEnum.preservationPlanning_aipStatus]: PreservationPlanningAipStatusStateModel | undefined;
  [StateEnum.preservationPlanning_sip]: PreservationPlanningSipStateModel | undefined;
  [StateEnum.preservationPlanning_dip]: PreservationPlanningDipStateModel | undefined;
}

@Injectable()
@State<PreservationPlanningStateModel>({
  name: StateEnum.preservationPlanning,
  defaults: {
    isLoadingCounter: 0,
    [StateEnum.preservationPlanning_deposit]: undefined,
    [StateEnum.preservationPlanning_monitoring]: undefined,
    [StateEnum.preservationPlanning_job]: undefined,
    [StateEnum.preservationPlanning_aipStatus]: undefined,
    [StateEnum.preservationPlanning_sip]: undefined,
    [StateEnum.preservationPlanning_dip]: undefined,
  },
  children: [
    PreservationPlanningDepositState,
    PreservationPlanningMonitoringState,
    PreservationPlanningJobState,
    PreservationPlanningAipStatusState,
    PreservationPlanningSipState,
    PreservationPlanningDipState,
  ],
})
export class PreservationPlanningState {
  constructor(protected store: Store) {
  }
}
