import {
  ChangeDetectionStrategy,
  Component,
  Inject,
  OnInit,
} from "@angular/core";
import {
  AbstractControl,
  FormBuilder,
  FormGroup,
  Validators,
} from "@angular/forms";
import {
  MAT_DIALOG_DATA,
  MatDialogRef,
} from "@angular/material/dialog";
import {Role} from "@models";
import {
  Actions,
  Store,
} from "@ngxs/store";
import {SharedAbstractDialog} from "@shared/components/dialogs/shared-abstract/shared-abstract.dialog";
import {BaseFormDefinition} from "@shared/models/base-form-definition.model";
import {SharedNotificationState} from "@shared/stores/notification/shared-notification.state";
import {SharedRoleAction} from "@shared/stores/role/shared-role.action";
import {SharedRoleState} from "@shared/stores/role/shared-role.state";
import {Observable} from "rxjs";
import {
  FormValidationHelper,
  MemoizedUtil,
  PropertyName,
  SolidifyValidator,
} from "solidify-frontend";

@Component({
  selector: "dlcm-preservation-space-organizational-unit-request-access-dialog",
  templateUrl: "./preservation-space-organizational-unit-request-access.dialog.html",
  styleUrls: ["./preservation-space-organizational-unit-request-access.dialog.scss"],
  changeDetection: ChangeDetectionStrategy.OnPush,
})
export class PreservationSpaceOrganizationalUnitRequestAccessDialog extends SharedAbstractDialog<PreservationSpaceOrganizationalUnitRequestAccessDialogData, OrgunitRequestAccessDialogResult> implements OnInit {
  isLoadingObs: Observable<boolean> = MemoizedUtil.isLoading(this._store, SharedNotificationState);
  listRoleObs: Observable<Role[]> = MemoizedUtil.list(this._store, SharedRoleState);

  form: FormGroup;
  formDefinition: FormComponentFormDefinition = new FormComponentFormDefinition();

  constructor(private readonly _store: Store,
              private readonly _actions$: Actions,
              protected readonly _dialogRef: MatDialogRef<PreservationSpaceOrganizationalUnitRequestAccessDialog>,
              private readonly _fb: FormBuilder,
              @Inject(MAT_DIALOG_DATA) public data: PreservationSpaceOrganizationalUnitRequestAccessDialogData,
  ) {
    super(_dialogRef, data);
  }

  ngOnInit(): void {
    super.ngOnInit();
    this._store.dispatch(new SharedRoleAction.GetAll());
    this.form = this._fb.group({
      [this.formDefinition.roleId]: ["", [Validators.required, SolidifyValidator]],
      [this.formDefinition.message]: ["", [Validators.required, SolidifyValidator]],
    });
  }

  onSubmit(): void {
    this.submit(this.form.value as OrgunitRequestAccessDialogResult);
  }

  getFormControl(key: string): AbstractControl {
    return FormValidationHelper.getFormControl(this.form, key);
  }

  get formValidationHelper(): typeof FormValidationHelper {
    return FormValidationHelper;
  }
}

class FormComponentFormDefinition extends BaseFormDefinition {
  @PropertyName() roleId: string;
  @PropertyName() message: string;
}

export interface PreservationSpaceOrganizationalUnitRequestAccessDialogData {
}

export interface OrgunitRequestAccessDialogResult {
  roleId: string;
  message: string;
}
