import {AdminArchiveAclStateModel} from "@admin/archive-acl/stores/admin-archive-acl.state";
import {
  ChangeDetectionStrategy,
  ChangeDetectorRef,
  Component,
  Injector,
  OnInit,
} from "@angular/core";
import {ActivatedRoute} from "@angular/router";
import {ArchiveACL} from "@models";
import {Navigate} from "@ngxs/router-plugin";
import {
  Actions,
  Select,
  Store,
} from "@ngxs/store";
import {preservationSpaceOrganizationalUnitArchiveAclActionNameSpace} from "@preservation-space/organizational-unit/stores/archive-acl/preservation-space-organizational-unit-archive-acl.action";
import {PreservationSpaceOrganizationalUnitArchiveAclState} from "@preservation-space/organizational-unit/stores/archive-acl/preservation-space-organizational-unit-archive-acl.state";
import {
  AppRoutesEnum,
  PreservationSpaceOrganizationalUnitRoutesEnum,
  RoutesEnum,
} from "@shared/enums/routes.enum";
import {StateEnum} from "@shared/enums/state.enum";
import {SharedArchiveAclAction} from "@shared/stores/archive-acl/shared-archive-acl.action";
import {SharedArchiveAclState} from "@shared/stores/archive-acl/shared-archive-acl.state";
import {Observable} from "rxjs";
import {
  AbstractCreateRoutable,
  MemoizedUtil,
  ModelFormControlEvent,
  StoreUtil,
  UrlQueryParamHelper,
} from "solidify-frontend";

@Component({
  selector: "dlcm-apreservation-space-organizational-unit-acl-create-routable",
  templateUrl: "./preservation-space-organizational-unit-archive-acl-create.routable.html",
  styleUrls: ["./preservation-space-organizational-unit-archive-acl-create.routable.scss"],
  changeDetection: ChangeDetectionStrategy.OnPush,
})
export class PreservationSpaceOrganizationalUnitArchiveAclCreateRoutable extends AbstractCreateRoutable<ArchiveACL, AdminArchiveAclStateModel> implements OnInit {
  isLoadingWithDependencyObs: Observable<boolean> = MemoizedUtil.isLoading(this._store, SharedArchiveAclState);
  @Select(PreservationSpaceOrganizationalUnitArchiveAclState.isReadyToBeDisplayedInCreateMode) isReadyToBeDisplayedInCreateModeObs: Observable<boolean>;
  orgUnitId: string;

  constructor(protected readonly _store: Store,
              protected readonly _actions$: Actions,
              protected readonly _changeDetector: ChangeDetectorRef,
              protected readonly _injector: Injector,
              protected readonly _route: ActivatedRoute) {
    super(_store, _actions$, _changeDetector, StateEnum.preservationSpace_organizationalUnit_archiveAcl, _injector, preservationSpaceOrganizationalUnitArchiveAclActionNameSpace as any, StateEnum.preservationSpace_organizationalUnit);
  }

  ngOnInit(): void {
    this.retrieveResIdFromUrl();
    this.urlQueryParameters = UrlQueryParamHelper.getQueryParamMappingObject();
  }

  protected retrieveResIdFromUrl(): void {
    this.orgUnitId = this._route.snapshot.parent.parent.paramMap.get(AppRoutesEnum.paramIdWithoutPrefixParam);
  }

  override create(modelFormControlEvent: ModelFormControlEvent<ArchiveACL>): void {
    super.saveInProgress();

    this.subscribe(StoreUtil.dispatchActionAndWaitForSubActionCompletion(this._store, this._actions$,
      new SharedArchiveAclAction.Create(modelFormControlEvent),
      SharedArchiveAclAction.CreateSuccess,
      resultAction => {
        this._store.dispatch(new Navigate([RoutesEnum.preservationSpaceOrganizationalUnitDetail, this.orgUnitId, PreservationSpaceOrganizationalUnitRoutesEnum.manageAcl, PreservationSpaceOrganizationalUnitRoutesEnum.manageAclDetail, resultAction.model.resId]));
      }));
  }

  override backToList(): void {
    this._store.dispatch(new Navigate([RoutesEnum.preservationSpaceOrganizationalUnitDetail, this.orgUnitId, PreservationSpaceOrganizationalUnitRoutesEnum.manageAcl]));
  }

  protected override cleanState(): void {
  }
}
