import {HttpClient} from "@angular/common/http";
import {Injectable} from "@angular/core";
import {ApiEnum} from "@app/shared/enums/api.enum";
import {
  AppCartDipAction,
  appCartDipActionNameSpace,
} from "@app/stores/cart/dip/app-cart-dip.action";
import {AppCartDipDataFileAction} from "@app/stores/cart/dip/data-file/app-cart-dip-data-file.action";
import {AppCartDipDataFileState} from "@app/stores/cart/dip/data-file/app-cart-dip-data-file.state";
import {Dip} from "@models";
import {
  Action,
  Actions,
  State,
  StateContext,
  Store,
} from "@ngxs/store";
import {LabelTranslateEnum} from "@shared/enums/label-translate.enum";
import {StateEnum} from "@shared/enums/state.enum";
import {Observable} from "rxjs/index";
import {
  ApiService,
  defaultResourceStateInitValue,
  DownloadService,
  NotificationService,
  ResourceState,
  ResourceStateModel,
  StoreUtil,
} from "solidify-frontend";
import {ApiActionNameEnum} from "../../../shared/enums/api-action-name.enum";

export interface AppCartDipStateModel extends ResourceStateModel<Dip> {
}

@Injectable()
@State<AppCartDipStateModel>({
  name: StateEnum.application_cart_dip,
  defaults: {
    ...defaultResourceStateInitValue(),
  },
  children: [
    AppCartDipDataFileState,
  ],
})
export class AppCartDipState extends ResourceState<AppCartDipStateModel, Dip> {
  constructor(protected apiService: ApiService,
              protected store: Store,
              protected notificationService: NotificationService,
              protected actions$: Actions,
              protected httpClient: HttpClient,
              private downloadService: DownloadService) {
    super(apiService, store, notificationService, actions$, {
      nameSpace: appCartDipActionNameSpace,
    });
  }

  protected get _urlResource(): string {
    return ApiEnum.accessDip;
  }

  @Action(AppCartDipAction.Download)
  download(ctx: StateContext<AppCartDipStateModel>, action: AppCartDipAction.Download): Observable<AppCartDipDataFileAction.GetAllSuccess> {
    const fileName = "dip_" + action.id + ".zip";
    this.notificationService.showInformation(LabelTranslateEnum.fileDownloadStart);

    return StoreUtil.dispatchActionAndWaitForSubActionCompletion(ctx, this.actions$,
      new AppCartDipDataFileAction.GetAll(action.id),
      AppCartDipDataFileAction.GetAllSuccess,
      resultAction => {
        const listDipDataFile = resultAction.list;
        let size = 0;
        listDipDataFile._data.forEach(dipDataFile => {
          size += dipDataFile.fileSize;
        });
        this.downloadService.download(false, `${this._urlResource}/${action.id}/${ApiActionNameEnum.DL}`, fileName, size);
      });
  }
}
