import {MatFormFieldAppearance} from "@angular/material/form-field";
import {ThemeEnum} from "@app/shared/enums/theme.enum";
import {Enums} from "@enums";
import {ViewModeTableEnum} from "@shared/enums/view-mode-table.enum";
import {DefaultSolidifyEnvironment} from "solidify-frontend";

export interface DlcmEnvironment extends DefaultSolidifyEnvironment {
  appTitle: string;
  appDescription: string;
  isDemoMode: boolean;
  maintenanceMode: boolean;
  ribbonEnabled: boolean;
  ribbonColor: string;
  ribbonText: string;

  carouselUrl: string | undefined;
  theme: ThemeEnum;
  defaultLanguage: Enums.Language.LanguageEnum;
  appLanguages: Enums.Language.LanguageEnum[];
  defaultPageSize: number;
  defaultPageSizeHomePage: number;
  defaultEnumValuePageSizeOption: number;
  defaultEnumValuePageSizeLazyLoad: number;
  pageSizeOptions: number[];
  defaultHomeOrgUnitSize: number;
  minimalPageSizeToRetrievePaginationInfo: number;
  maximalPageSizeToRetrievePaginationInfo: number;
  useSelfHostedStreamSaverMitm: boolean; // If false, use fallback service worker on github. If true, use local service worker and allow to avoid message downloaded from https://jimmywarting.github.io/StreamSaver.js
  computeOAuthEndpoint: boolean;
  googleAnalyticsId: string | undefined;
  googleAnalyticsLearnMorePage: string;
  appearanceInputMaterial: MatFormFieldAppearance;
  defaultStickyDatatableHeight: number;
  displayPrivacyPolicyAndTermsOfUseApprovalDialog: boolean;
  privacyPolicyLink: string;
  termsOfUseLink: string;

  // Base URL For Module Applications (override by AppAction.LoadModules)
  preservation_planning?: string | undefined;
  authorization?: string | undefined;
  admin: string;
  ingest?: string | undefined;
  preingest?: string | undefined;
  access?: string | undefined;
  archivalStorage?: string[] | undefined;
  dataManagement?: string | undefined;

  defaultStorageIndex: number;
  archivalStorageName: string[];

  // Documentation
  documentationTocApisGuide: string | undefined;
  documentationTocIntegrationGuide: string | undefined;
  documentationTocToolsGuide: string | undefined;
  documentationTocUserGuidePath: string | undefined;
  documentationTocUserGuide: string | undefined;

  // OAuth2 properties
  tokenInMemoryStorage: boolean;
  reloadAfterLogout: boolean;

  // Orcid properties
  orcidQueryParam: string;
  orcidAuthorizeUrl: string;
  orcidUrl: string;

  // Access request
  accessRequestRoute: string;

  // Breakpoints
  breakpointXs: string;
  breakpointSm: string;
  breakpointMd: string;
  breakpointLg: string;

  // Others
  modalHeight: string;
  modalWidth: string;
  showDebugInformation: boolean;
  institutionUrl: string;

  // Spinner
  defaultSpinnerStrokeWidth: number;
  defaultSpinnerDiameter: number;
  defaultButtonSpinnerStrokeWidth: number;
  defaultButtonSpinnerDiameter: number;

  // Home
  defaultHomeViewModeTableEnum: ViewModeTableEnum;

  // Tweeter
  twitterAccount: string;
  twitterTweetToDisplay: number;

  // Carousel
  frequencyChangeCarouselTileInSecond: number;

  urlNationalArchivePronom: string;

  // Tooltip duration
  timeBeforeDisplayTooltipOnDataTable: number;

  doiLink: string;

  // Polling
  refreshTabStatusCounterIntervalInSecond: number;
  refreshOrderAvailableIntervalInSecond: number;
  refreshNotificationInboxAvailableIntervalInSecond: number;
  refreshDepositSubmittedIntervalInSecond: number;
  pollingMaxIntervalInSecond: number;

  cdkDropListIdPrefix: string;
  classInputIgnored: string;
  classInputHide: string;
}
